using System;
using System.Collections;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Security.Cryptography;
using System.Text;
using System.Web.Configuration;
using System.Xml;
using DotNetSCORM.DataTools;


namespace ClubSite.API
{

    /// -----------------------------------------------------------------------------
    /// <summary>
    /// The Config class provides access to the web.config file
    /// </summary>
    /// <remarks>
    /// </remarks>
    /// <history>
    ///		
    /// </history>
    /// -----------------------------------------------------------------------------
    public class Config
    {

        #region "Private Methods"
        private static string CreateKey(int numBytes)
        {
            RNGCryptoServiceProvider rng = new RNGCryptoServiceProvider();
            byte[] buff = new byte[numBytes];

            rng.GetBytes(buff);

            return BytesToHexString(buff);
        }

        private static string BytesToHexString(byte[] bytes)
        {
            StringBuilder hexString = new StringBuilder(64);

            int counter;
            for (counter = 0; counter <= bytes.Length - 1; counter++)
            {
                hexString.Append(String.Format("{0:X2}", bytes[counter]));
            }

            return hexString.ToString();
        }

        #endregion


        #region "Shared Methods"

        public static XmlDocument AddAppSetting(XmlDocument xmlDoc, string Key, string Value)
        {

            XmlElement xmlElement;

            // retrieve the appSettings node 
            XmlNode xmlAppSettings = xmlDoc.SelectSingleNode("//appSettings");

            if ((xmlAppSettings != null))
            {
                // get the node based on key
                XmlNode xmlNode = xmlAppSettings.SelectSingleNode(("//add[@key='" + Key + "']"));

                if ((xmlNode != null))
                {
                    // update the existing element
                    xmlElement = (XmlElement)xmlNode;
                    xmlElement.SetAttribute("value", Value);
                }
                else
                {
                    // create a new element
                    xmlElement = xmlDoc.CreateElement("add");
                    xmlElement.SetAttribute("key", Key);
                    xmlElement.SetAttribute("value", Value);
                    xmlAppSettings.AppendChild(xmlElement);
                }
            }

            // return the xml doc
            return xmlDoc;

        }


        public static void BackupConfig()
        {
            string backupFolder = Globals.glbConfigFolder + "Backup_" + DateTime.Now.Year.ToString() + DateTime.Now.Month.ToString() + DateTime.Now.Day.ToString() + DateTime.Now.Hour.ToString() + DateTime.Now.Minute.ToString() + "\\";

            //save the current config files
            try
            {
                if (!Directory.Exists(ApplicationMapPath() + backupFolder))
                {
                    Directory.CreateDirectory(ApplicationMapPath() + backupFolder);
                }

                if (File.Exists(ApplicationMapPath() + "\\web.config"))
                {
                    File.Copy(ApplicationMapPath() + "\\web.config", ApplicationMapPath() + backupFolder + "web_old.config", true);
                }
            }
            catch (Exception ex)
            {
                //Error backing up old web.config 
                //This error is not critical, so can be ignored
            }

        }

        public static string GetSetting(string setting)
        {
            return WebConfigurationManager.AppSettings[setting];
        }

        public static object GetSection(string section)
        {
            return WebConfigurationManager.GetWebApplicationSection(section);
        }

        public static XmlDocument Load()
        {
            return Load("web.config");
        }

        public static XmlDocument Load(string filename)
        {
            // open the config file
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(ApplicationMapPath() + "\\" + filename);
            // test for namespace added by Web Admin Tool
            if (xmlDoc.DocumentElement.GetAttribute("xmlns") != "")
            {
                // remove namespace
                string strDoc = xmlDoc.InnerXml.Replace("xmlns=\"http://schemas.microsoft.com/.NetConfiguration/v2.0\"", "");
                xmlDoc.LoadXml(strDoc);
            }
            return xmlDoc;
        }


        public static string Save(XmlDocument xmlDoc)
        {
            return Save(xmlDoc, "web.config");
        }

        public static string Save(XmlDocument xmlDoc, string filename)
        {
            try
            {
                string strFilePath = ApplicationMapPath() + "\\" + filename;
                FileAttributes objFileAttributes = FileAttributes.Normal;
                if (File.Exists(strFilePath))
                {
                    // save current file attributes
                    objFileAttributes = File.GetAttributes(strFilePath);
                    // change to normal ( in case it is flagged as read-only )
                    File.SetAttributes(strFilePath, FileAttributes.Normal);
                }
                // save the config file
                XmlTextWriter writer = new XmlTextWriter(strFilePath, null);
                writer.Formatting = Formatting.Indented;
                xmlDoc.WriteTo(writer);
                writer.Flush();
                writer.Close();
                // reset file attributes
                File.SetAttributes(strFilePath, objFileAttributes);
                return "";
            }
            catch (Exception exc)
            {
                // the file permissions may not be set properly
                return exc.Message;
            }
        }

        public static void Touch()
        {
            File.SetLastWriteTime(ApplicationMapPath() + "\\web.config", System.DateTime.Now);
        }

        public static void UpdateConnectionString(string conn)
        {

            XmlDocument xmlConfig = Load();
            string name = Globals.glbDotNetSCORMDB;

            //Update ConnectionStrings
            XmlNode xmlConnection = xmlConfig.SelectSingleNode("configuration/connectionStrings/add[@name='" + name + "']");
            XmlUtils.UpdateAttribute(xmlConnection, "connectionString", conn);

            //Update AppSetting
            XmlNode xmlAppSetting = xmlConfig.SelectSingleNode("configuration/appSettings/add[@key='" + name + "']");
            XmlUtils.UpdateAttribute(xmlAppSetting, "value", conn);

            //Save changes
            Save(xmlConfig);

        }


        public static string UpdateMachineKey()
        {
            string appRoot = Settings.ApplicationRoot();
            string backupFolder = appRoot + "Backup_" + DateTime.Now.Year.ToString() + DateTime.Now.Month.ToString() + DateTime.Now.Day.ToString() + DateTime.Now.Hour.ToString() + DateTime.Now.Minute.ToString() + "\\";
            XmlDocument xmlConfig = new XmlDocument();
            string strError = "";

            //save the current config files
            BackupConfig();

            try
            {
                // open the web.config
                xmlConfig = Load();

                // create random keys for the Membership machine keys
                xmlConfig = UpdateMachineKey(xmlConfig);
            }
            catch (Exception ex)
            {
                strError += ex.Message;
            }

            // save a copy of the web.config
            strError += Save(xmlConfig, backupFolder + "web_.config");

            // save the web.config
            strError += Save(xmlConfig);

            return strError;
        }

        public static XmlDocument UpdateMachineKey(XmlDocument xmlConfig)
        {

            string validationKey = CreateKey(20);
            string decryptionKey = CreateKey(24);

            XmlNode xmlMachineKey = xmlConfig.SelectSingleNode("configuration/system.web/machineKey");
            XmlUtils.UpdateAttribute(xmlMachineKey, "validationKey", validationKey);
            XmlUtils.UpdateAttribute(xmlMachineKey, "decryptionKey", decryptionKey);

            xmlConfig = AddAppSetting(xmlConfig, "InstallationDate", System.DateTime.Today.ToShortDateString());

            return xmlConfig;

        }

        public static string UpdateValidationKey()
        {
            string backupFolder = Globals.glbConfigFolder + "Backup_" + DateTime.Now.Year.ToString() + DateTime.Now.Month.ToString() + DateTime.Now.Day.ToString() + DateTime.Now.Hour.ToString() + DateTime.Now.Minute.ToString() + "\\";
            XmlDocument xmlConfig = new XmlDocument();
            string strError = "";

            //save the current config files
            BackupConfig();

            try
            {
                // open the web.config
                xmlConfig = Load();

                // create random keys for the Membership machine keys
                xmlConfig = UpdateValidationkey(xmlConfig);
            }
            catch (Exception ex)
            {
                strError += ex.Message;
            }

            // save a copy of the web.config
            strError += Save(xmlConfig, backupFolder + "web_.config");

            // save the web.config
            strError += Save(xmlConfig);

            return strError;
        }

        public static XmlDocument UpdateValidationkey(XmlDocument xmlConfig)
        {
            XmlNode xmlMachineKey;
            string strError = string.Empty;

            xmlMachineKey = xmlConfig.SelectSingleNode("configuration/system.web/machineKey");

            if (xmlMachineKey.Attributes["validationKey"].Value == "F9D1A2D3E1D3E2F7B3D9F90FF3965ABDAC304902")
            {
                string validationKey = CreateKey(20);
                XmlUtils.UpdateAttribute(xmlMachineKey, "validationKey", validationKey);
            }

            return xmlConfig;
        }


        public static string ApplicationMapPath()
        {
            string pathStr;

            pathStr = System.AppDomain.CurrentDomain.BaseDirectory.Substring(0, System.AppDomain.CurrentDomain.BaseDirectory.Length - 1);
            pathStr = pathStr.Replace("/", "\\");

            return pathStr;

        }

        #endregion

    }

}
