using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Web;
using System.Web.Management;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;
using ClubSite.API;
using DotNetSCORM.DataTools;

namespace DotNetSCORM.Install
{

    /// <summary>
    /// Summary description for InstallUtil
    /// </summary>
    public class InstallUtil
    {
        public InstallUtil()
        {

        }

        #region "Private Variables"

        private string _connectionString;
        private string _database;


        #endregion

        #region "Properties"


        public string ConnectionString
        {
            get { return _connectionString; }
            set { _connectionString = value; }
        }

        public string Database
        {
            get { return _database; }
            set { _database = value; }
        }


        #endregion


        #region "Public Methods"

        public string CheckConnection()
        {


            return "Check Connection Needs to be coded up";
        }

        public void InstallDNS()
        {
            //Performs the actual Installation activity
            string strError = "";
             string strHostPath = Config.ApplicationMapPath();
            XmlDocument xmlDoc = new XmlDocument();

            //Get the Installation Template
            try
            {
            xmlDoc.Load(HttpContext.Current.Server.MapPath("~/Install/Install.config"));
            }
            catch
            {
                strError = "Error Loading Template";
            }

            if (strError == "")
            {
                //TODO: Beef up the error checking
                WriteText(HttpContext.Current.Response, "<h2>Installation in Progress</h2>");
                InstallAppSQL(xmlDoc);
                CreateRoles(xmlDoc);
                CreateAdmin(xmlDoc);
                CreateWelcomeMsg(xmlDoc);
                WriteText(HttpContext.Current.Response, "<br><h2>DotNetSCORM Installation Complete</h2><br>");
                

            }
            else
            {
                //We could not load the template, return 500 Error
                HttpContext.Current.Response.Clear();
                HttpContext.Current.Response.Redirect("~/ErrorPage.aspx?error=" + strError);
            }
        }


        public bool InstallAppSQL(XmlDocument xmlDoc)
        {
            //TODO: Beef up error reporting code
            string resultString = "";
            string strScript = "";
            string strScriptName = "";
            XmlNode node;

            node = xmlDoc.SelectSingleNode("//dotnetscorm/database");
            if (node != null)
            {
                _database = node.InnerText;
                _connectionString = Settings.ConnectionString();

                WriteText(HttpContext.Current.Response, "Installing Membership and Roles<br>");
                resultString = InstallMembership();

                WriteText(HttpContext.Current.Response, "Installing Schema and other SQL Scripts<br>");
                node = xmlDoc.SelectSingleNode("//dotnetscorm/scripts");
                if (node != null)
                {
                    //Loop through the scripts
                    foreach (XmlNode ScriptNode in node.SelectNodes("script"))
                    {
                        strScript = HttpContext.Current.Server.MapPath(ScriptNode.InnerText);
                        strScriptName = ScriptNode.Attributes["name"].Value;
                        ExecuteScript(strScript, strScriptName);
                    }
                }
            }

            return true;
        }

        private string InstallMembership()
        {
            // Install Membership and Roles.
            string ErrorString = "";

            try
            {
                SqlServices.Install(_database, SqlFeatures.Membership, _connectionString);
                SqlServices.Install(_database, SqlFeatures.RoleManager, _connectionString);
            }
            catch
            {
                ErrorString = "Error Installing Membership";
            }
            return ErrorString;
        }

        private string CreateRoles(XmlDocument XmlDoc)
        {
            // TODO: Can we figure out how to Wordsmith this and place the roles into the Install Template?
            WriteText(HttpContext.Current.Response, "Installing Roles");
            ExecuteScript("EXEC aspnet_Roles_CreateRole 'DotNetSCORM', 'Administrator'");
            ExecuteScript("EXEC aspnet_Roles_CreateRole 'DotNetSCORM', 'Learner'");
            WriteText(HttpContext.Current.Response, " - Done<br>");
            return "";
        }

        private string CreateAdmin(XmlDocument XmlDoc)
        {
            XmlNode node;
            string username = "";
            string password;
            string email;
            string firstname;
            string lastname;

            node = XmlDoc.SelectSingleNode("//dotnetscorm/adminuser");

                if ((node != null))
                {
                    WriteText(HttpContext.Current.Response, "Configuring Admin");

                    //Parse the AdminUser nodes 
                    firstname = XmlUtils.GetNodeValue(node, "firstname", null);
                    lastname = XmlUtils.GetNodeValue(node, "lastname", null);
                    username = XmlUtils.GetNodeValue(node, "username", null);
                    password = XmlUtils.GetNodeValue(node, "password", null);
                    email = XmlUtils.GetNodeValue(node, "email", null);

                    try
                    {
                        ClubSite.API.DataSetTableAdapters.MemberInfoTableAdapter da = new ClubSite.API.DataSetTableAdapters.MemberInfoTableAdapter();
                        MembershipUser user = Membership.CreateUser(username, password, email);
                        da.Insert(((Guid)(user.ProviderUserKey)), "", "", firstname, lastname);
                        user.IsApproved = true;
                            // Add the user to the role
                        Roles.AddUserToRole(user.UserName, "Administrator");
                        Membership.UpdateUser(user);
                        FormsAuthentication.SetAuthCookie(username, false);
                        WriteText(HttpContext.Current.Response, " - Done<br>");


                    }
                    catch
                    {
                    }

                }
                return username;
        }

        private string CreateWelcomeMsg(XmlDocument XmlDoc)
        {
            XmlNode node;
            string username = "";
            string password;

            string msgText;

            node = XmlDoc.SelectSingleNode("//dotnetscorm/adminuser");

            if ((node != null))
            {
                WriteText(HttpContext.Current.Response, "Configuring Install Event Msg");

                //Parse the AdminUser nodes 
                username = XmlUtils.GetNodeValue(node, "username", null);
                password = XmlUtils.GetNodeValue(node, "password", null);

                try
                {

                    StreamReader sReader = new StreamReader(ApplicationMapPath() + "\\Install\\WelcomeMsg.htm");

                    //Read the first Line
                    msgText = sReader.ReadLine();
                    //Read the rest of the msg
                    while (sReader != null)
                    {
                        msgText += sReader.ReadLine();
                    }
                    sReader.Close();
                    
                    //Create the Msg
                    StringBuilder sb = new StringBuilder(msgText);
                    sb.Replace("$username$", username);
                    sb.Replace("$password$", password);

                    msgText = sb.ToString();

                    //TODO: We need to put these into the Database
                    //Write the simple Welcome Msg
                    StreamWriter sWriter = new StreamWriter(ApplicationMapPath() + "\\WelcomeMsg.htm");

                    sWriter.Write(msgText);
                    sWriter.Close();

                    WriteText(HttpContext.Current.Response, " - Done<br>");


                }
                catch
                {
                }

            }
            return username;
        }


        
        private string ExecuteScript(string SQLFile, string scriptName)
        {
            //TODO: Beef up error reporting code
            string ErrorString = string.Empty;
            SqlConnection conn = null;

            WriteText(HttpContext.Current.Response, "Installing Script " + scriptName);

            try
            {
                //Open the Schema File, read it and install it.
                using (StreamReader sr = new StreamReader(SQLFile))
                {

                    // Create new connection to database
                    conn = new SqlConnection(_connectionString);

                    conn.Open();

                    while (!sr.EndOfStream)
                    {
                        StringBuilder sb = new StringBuilder();
                        SqlCommand cmd = conn.CreateCommand();

                        while (!sr.EndOfStream)
                        {
                            string s = sr.ReadLine();
                            if (s != null && s.ToUpper().Trim().Equals("GO"))
                            {
                                break;
                            }

                            sb.AppendLine(s);
                        }

                        // Execute T-SQL against the target database
                        cmd.CommandText = sb.ToString();
                        cmd.CommandTimeout = 600;

                        cmd.ExecuteNonQuery();
                    }
                    WriteText(HttpContext.Current.Response, " - Success<br>");
                }
            }
            catch
            {
                WriteText(HttpContext.Current.Response, " - Error<br>");
                ErrorString = "Error Installing Schema";
            }
            finally
            {
                //Clos Out the Connection
                if (conn != null)
                {
                    try
                    {
                        conn.Close();
                        conn.Dispose();
                    }
                    catch (Exception e)
                    {
                        ErrorString = String.Format(@"Could not close the connection.  Error was {0}", e.ToString());
                    }
                }

            }
            return ErrorString;

        }

        private string ExecuteScript(string sCmd)
        {
            string ErrorString = "";
            SqlConnection conn = new SqlConnection();

            try
            {
                // Create new connection to database
                conn = new SqlConnection(_connectionString);
                conn.Open();
                SqlCommand cmd = conn.CreateCommand();
                cmd.CommandText = sCmd;
                cmd.CommandTimeout = 600;
                cmd.ExecuteNonQuery();

            }
            catch
            {

            }
            finally
            {
                //Clos Out the Connection
                if (conn != null)
                {
                    try
                    {
                        conn.Close();
                        conn.Dispose();
                    }
                    catch (Exception e)
                    {
                        ErrorString = String.Format(@"Could not close the connection.  Error was {0}", e.ToString());
                    }
                }

            }
            return ErrorString;
        }

        public static string UpdateMachineKey()
        {
            string appRoot = Settings.ApplicationRoot();
            string backupFolder = appRoot + "Backup_" + DateTime.Now.Year.ToString() + DateTime.Now.Month.ToString() + DateTime.Now.Day.ToString() + DateTime.Now.Hour.ToString() + DateTime.Now.Minute.ToString() + "\\";
            XmlDocument xmlConfig = new XmlDocument();
            string strError = "";

            //save the current config files
            BackupConfig();

            try
            {
                // open the web.config
                xmlConfig = Load();

                // create random keys for the Membership machine keys
                xmlConfig = UpdateMachineKey(xmlConfig);
            }
            catch (Exception ex)
            {
                strError += ex.Message;
            }

            // save the web.config
            strError += Save(xmlConfig);

            return strError;
        }


        public static XmlDocument UpdateMachineKey(XmlDocument xmlConfig)
        {

            string validationKey = CreateKey(64);
            string decryptionKey = CreateKey(33);

            XmlNode xmlMachineKey = xmlConfig.SelectSingleNode("configuration/system.web/machineKey");
            XmlUtils.UpdateAttribute(xmlMachineKey, "validationKey", validationKey);
            XmlUtils.UpdateAttribute(xmlMachineKey, "decryptionKey", decryptionKey);

            xmlConfig = AddAppSetting(xmlConfig, "InstallationDate", System.DateTime.Today.ToShortDateString());

            return xmlConfig;

        }

        public static XmlDocument AddAppSetting(XmlDocument xmlDoc, string Key, string Value)
        {

            XmlElement xmlElement;

            // retrieve the appSettings node 
            XmlNode xmlAppSettings = xmlDoc.SelectSingleNode("//appSettings");

            if ((xmlAppSettings != null))
            {
                // get the node based on key
                XmlNode xmlNode = xmlAppSettings.SelectSingleNode(("//add[@key='" + Key + "']"));

                if ((xmlNode != null))
                {
                    // update the existing element
                    xmlElement = (XmlElement)xmlNode;
                    xmlElement.SetAttribute("value", Value);
                }
                else
                {
                    // create a new element
                    xmlElement = xmlDoc.CreateElement("add");
                    xmlElement.SetAttribute("key", Key);
                    xmlElement.SetAttribute("value", Value);
                    xmlAppSettings.AppendChild(xmlElement);
                }
            }

            // return the xml doc
            return xmlDoc;

        }

        public static void BackupConfig()
        {
            string backupFolder = Globals.glbConfigFolder + "Backup_" + DateTime.Now.Year.ToString() + DateTime.Now.Month.ToString() + DateTime.Now.Day.ToString() + DateTime.Now.Hour.ToString() + DateTime.Now.Minute.ToString() + "\\";

            //save the current config files
            try
            {
                if (!Directory.Exists(ApplicationMapPath() + backupFolder))
                {
                    Directory.CreateDirectory(ApplicationMapPath() + backupFolder);
                }

                if (File.Exists(ApplicationMapPath() + "\\web.config"))
                {
                    File.Copy(ApplicationMapPath() + "\\web.config", ApplicationMapPath() + backupFolder + "web_old.config", true);
                }
            }
            catch (Exception ex)
            {
                //Error backing up old web.config 
                //This error is not critical, so can be ignored
            }

        }

        public static XmlDocument Load()
        {
            return Load("web.config");
        }

        public static XmlDocument Load(string filename)
        {
            // open the config file
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(ApplicationMapPath() + "\\" + filename);
            // test for namespace added by Web Admin Tool
            if (xmlDoc.DocumentElement.GetAttribute("xmlns") != "")
            {
                // remove namespace
                string strDoc = xmlDoc.InnerXml.Replace("xmlns=\"http://schemas.microsoft.com/.NetConfiguration/v2.0\"", "");
                xmlDoc.LoadXml(strDoc);
            }
            return xmlDoc;
        }


        public static string Save(XmlDocument xmlDoc)
        {
            return Save(xmlDoc, "web.config");
        }

        public static string Save(XmlDocument xmlDoc, string filename)
        {
            try
            {
                string strFilePath = ApplicationMapPath() + "\\" + filename;
                FileAttributes objFileAttributes = FileAttributes.Normal;
                if (File.Exists(strFilePath))
                {
                    // save current file attributes
                    objFileAttributes = File.GetAttributes(strFilePath);
                    // change to normal ( in case it is flagged as read-only )
                    File.SetAttributes(strFilePath, FileAttributes.Normal);
                }
                // save the config file
                XmlTextWriter writer = new XmlTextWriter(strFilePath, null);
                writer.Formatting = Formatting.Indented;
                xmlDoc.WriteTo(writer);
                writer.Flush();
                writer.Close();
                // reset file attributes
                File.SetAttributes(strFilePath, objFileAttributes);
                return "";
            }
            catch (Exception exc)
            {
                // the file permissions may not be set properly
                return exc.Message;
            }
        }




        public static string CreateKey(int numBytes)
        {
            RNGCryptoServiceProvider rng = new RNGCryptoServiceProvider();
            byte[] buff = new byte[numBytes - 1];

            rng.GetBytes(buff);

            return BytesToHexString(buff);
        }


        private static string BytesToHexString(byte[] bytes)
        {
            StringBuilder hexString = new StringBuilder(64);

            int counter;
            for (counter = 0; counter <= bytes.Length - 1; counter++)
            {
                hexString.Append(String.Format("{0:X2}", bytes[counter]));
            }

            return hexString.ToString();
        }


        public static string ApplicationMapPath()
        {
            string pathStr;

            pathStr = System.AppDomain.CurrentDomain.BaseDirectory.Substring(0, System.AppDomain.CurrentDomain.BaseDirectory.Length - 1);
            pathStr = pathStr.Replace("/", "\\");

            return pathStr;

        }


        /// -----------------------------------------------------------------------------
        /// <summary>
        /// WriteHeader outputs the Header during Install/Upgrade etc
        /// </summary>
        /// <remarks>
        /// </remarks>
        /// <param name="response">The ASP.Net Response object</param>
        /// <param name="mode">The mode Install/Upgrade etc</param>
        /// <history>
        ///		
        /// </history>
        /// -----------------------------------------------------------------------------
        public void WriteHeader(HttpResponse response, string mode)
        {
            //Set Response buffer to False
            response.Buffer = false;

            // read install page and insert into response stream
            if (File.Exists(System.Web.HttpContext.Current.Server.MapPath("~/Install/Install.htm")))
            {
                StreamReader objStreamReader;
                objStreamReader = File.OpenText(System.Web.HttpContext.Current.Server.MapPath("~/Install/Install.htm"));
                string strHTML = objStreamReader.ReadToEnd();
                objStreamReader.Close();
                response.Write(strHTML);
            }

            switch (mode)
            {
                case "install":
                    response.Write("<h1>Installing DotNetSCORM</h1>");
                    break;
                case "upgrade":
                    response.Write("<h1>Upgrading DotNetSCORM</h1>");
                    break;
                case "installResources":
                    response.Write("<h1>Installing Resources</h1>");
                    break;
                case "executeScripts":
                    response.Write("<h1>Executing Scripts</h1>");
                    break;
                case "none":
                    response.Write("<h1>Nothing To Install At This Time</h1>");
                    break;
                case "error":
                    response.Write("<h1>Error Installing DotNetSCORM</h1>");
                    break;
                default:
                    response.Write("<h1>" + mode + "</h1>");
                    break;
            }
            response.Flush();
        }


        /// -----------------------------------------------------------------------------
        /// <summary>
        /// WriteFooter outputs the Footer during Install/Upgrade etc
        /// </summary>
        /// <remarks>
        /// </remarks>
        /// <param name="response">The ASP.Net Response object</param>
        /// <history>
        ///		[cnurse]	02/21/2005	created
        /// </history>
        /// -----------------------------------------------------------------------------
        public void WriteFooter(HttpResponse response)
        {

            response.Write("</body>");
            response.Write("</html>");
            response.Flush();

        }

        public static void WriteText(HttpResponse response, string strText)
        {
            response.Write(strText);
            response.Flush();
        }



        #endregion





    }

}


