using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Microsoft.LearningComponents;
using Microsoft.LearningComponents.Storage;
using DotNetSCORM.LearningAPI;

using Schema = DotNetSCORM.LearningAPI.Schema;




public partial class DNSControls_UploadContent : DotNetSCORM.LearningAPI.DotNetSCORMControlBase
{
    public Boolean UploadComplete = false;

    protected void Page_Load(object sender, EventArgs e)
    {



    }
    protected void UploadPackageButton_OnClick(object sender, EventArgs e)
    {
        // the user clicked "Upload"...

        // do nothing if the user didn't select a file to upload
        if (!UploadedPackageFile.HasFile)
            return;

        // hide the upload panel and show the message panel; the message panel will hold
        // information about the success or failure of the package upload operation
        UploadPanel.Visible = false;
        MessagePanel.Visible = true;

        // attempt to import the uploaded file into PackageStore
        try
        {
            // set <currentUser> to information about the current user; we need the current user's
            // UserItemIdentifier
            DotNetSCORM.LearningAPI.LStoreUserInfo currentUser = GetCurrentUserInfo();

            // import the package file; set packageId to the ID of the uploaded
            // package; set importLog to a collection of warnings (if any)
            // about the import process
            PackageItemIdentifier packageId;
            ValidationResults importLog;
            using (PackageReader packageReader = PackageReader.Create(UploadedPackageFile.FileContent))
            {
                // Add package, asking to fix anything that can be fixed.
                AddPackageResult result = PStore.AddPackage(packageReader, new PackageEnforcement(false, false, false));
                packageId = result.PackageId;
                importLog = result.Log;
            }

            // fill in the application-specific columns of the PackageItem table
            LearningStoreJob job = LStore.CreateJob();
            Dictionary<string, object> properties = new Dictionary<string, object>();
            properties[Schema.PackageItem.Owner] = currentUser.Id;
            properties[Schema.PackageItem.FileName] = UploadedPackageFile.FileName;
            properties[Schema.PackageItem.UploadDateTime] = DateTime.Now;
            job.UpdateItem(packageId, properties);
            job.Execute();

            // retrieve information about the package
            job = LStore.CreateJob();
            RequestMyTraining(job, packageId);
            //GetMyTrainingResultsToHtml(job.Execute<DataTable>(), TrainingGrid);
            //TODO: Wire up the inforamtion Page about the uploaded content
            job.Execute<DataTable>();



            // provide user feedback
            if (importLog.HasWarnings)
            {
                // display warnings
                WarningIntro.Visible = true;
                WarningMessages.Visible = true;
                foreach (ValidationResult result in importLog.Results)
                    ScrollingMessagesList.Items.Add(new ListItem(result.Message));
            }
            else
            {
                // the operation was successful, and there are no messages to display to the user,
                // so close the dialog
                //CloseDialogScript.Visible = true;
                //TODO: Display the information about the uploaded package
            }
        }
        catch (PackageImportException ex)
        {
            // a package import failure occurred -- display the error message
            ErrorIntro.Visible = true;
            ErrorMessage.Visible = true;



            foreach (ValidationResult result in ex.Log.Results)
                ErrorMessageScrollingList.Items.Add(new ListItem(result.Message));

            if (ex.InnerException != null)
                ErrorMessageScrollingList.Items.Add(new ListItem(
                            Server.HtmlEncode(ex.InnerException.Message)));

        }
        catch (Exception ex)
        {
            // an unexpected error occurred -- display a generic message that doesn't include the
            // exception message (since that message may include sensitive information), and write
            // the exception message to the event log
            ErrorIntro.Visible = true;
            ErrorMessage.Visible = true;
            ErrorMessage.Controls.Add(new System.Web.UI.LiteralControl(
                Server.HtmlEncode("A serious error occurred.  Please contact your system administrator.  More information has been written to the server event log.")));
            LogEvent(System.Diagnostics.EventLogEntryType.Error,
                "An exception occurred while uploading a package:\n\n{0}\n\n", ex.ToString());
        }

    }
    protected void UploadDone_Click(object sender, EventArgs e)
    {
        UploadComplete = true;
    }
}
